@@ -49,6 +49,7 @@ class Tile;
 class Tileset;
 class TilesetFormat;
 class Terrain;
+class WangSet;
 
 typedef QSharedPointer<Tileset> SharedTileset;
 
@@ -161,14 +162,15 @@ public:
 
     void setImageReference(const ImageReference &reference);
 
-    bool loadFromImage(const QImage &image, const QString &fileName);
+    bool loadFromImage(const QImage &image, const QUrl &source);
+    bool loadFromImage(const QImage &image, const QString &source);
     bool loadFromImage(const QString &fileName);
     bool loadImage();
 
     SharedTileset findSimilarTileset(const QVector<SharedTileset> &tilesets) const;
 
-    const QString &imageSource() const;
-    void setImageSource(const QString &imageSource);
+    const QUrl &imageSource() const;
+    void setImageSource(const QUrl &imageSource);
     bool isCollection() const;
 
     int columnCountForWidth(int width) const;
@@ -181,10 +183,20 @@ public:
     Terrain *addTerrain(const QString &name, int imageTileId);
     void insertTerrain(int index, Terrain *terrain);
     Terrain *takeTerrainAt(int index);
+    void swapTerrains(int index, int swapIndex);
 
     int terrainTransitionPenalty(int terrainType0, int terrainType1) const;
+    int maximumTerrainDistance() const;
 
-    Tile *addTile(const QPixmap &image, const QString &source = QString());
+    const QList<WangSet*> &wangSets() const;
+    int wangSetCount() const;
+    WangSet *wangSet(int index) const;
+
+    void addWangSet(WangSet *wangSet);
+    void insertWangSet(int index, WangSet *wangSet);
+    WangSet *takeWangSetAt(int index);
+
+    Tile *addTile(const QPixmap &image, const QUrl &source = QUrl());
     void addTiles(const QList<Tile*> &tiles);
     void removeTiles(const QList<Tile *> &tiles);
     void deleteTile(int id);
@@ -195,15 +207,16 @@ public:
 
     void setTileImage(Tile *tile,
                       const QPixmap &image,
-                      const QString &source = QString());
+                      const QUrl &source = QUrl());
 
     void markTerrainDistancesDirty();
 
     SharedTileset sharedPointer() const;
 
-    void setLoaded(bool loaded);
-    bool loaded() const;
-    bool imageLoaded() const;
+    void setStatus(LoadingStatus status);
+    void setImageStatus(LoadingStatus status);
+    LoadingStatus status() const;
+    LoadingStatus imageStatus() const;
 
     void swap(Tileset &other);
 
@@ -246,8 +259,10 @@ private:
     QMap<int, Tile*> mTiles;
     int mNextTileId;
     QList<Terrain*> mTerrainTypes;
+    QList<WangSet*> mWangSets;
+    int mMaximumTerrainDistance;
     bool mTerrainDistancesDirty;
-    bool mLoaded;
+    LoadingStatus mStatus;
     QColor mBackgroundColor;
     QPointer<TilesetFormat> mFormat;
 
@@ -507,15 +522,15 @@ inline void Tileset::setBackgroundColor(QColor color)
  */
 inline bool Tileset::loadFromImage(const QString &fileName)
 {
-    return loadFromImage(QImage(fileName), fileName);
+    return loadFromImage(QImage(fileName), QUrl::fromLocalFile(fileName));
 }
 
 /**
- * Returns the file name of the external image that contains the tiles in
+ * Returns the URL of the external image that contains the tiles in
  * this tileset. Is an empty string when this tileset doesn't have a
  * tileset image.
  */
-inline const QString &Tileset::imageSource() const
+inline const QUrl &Tileset::imageSource() const
 {
     return mImageReference.source;
 }
@@ -553,6 +568,21 @@ inline Terrain *Tileset::terrain(int index) const
     return index >= 0 ? mTerrainTypes[index] : nullptr;
 }
 
+inline const QList<WangSet*> &Tileset::wangSets() const
+{
+    return mWangSets;
+}
+
+inline int Tileset::wangSetCount() const
+{
+    return mWangSets.size();
+}
+
+inline WangSet *Tileset::wangSet(int index) const
+{
+    return index >= 0 ? mWangSets[index] : nullptr;
+}
+
 /**
  * Sets the next id to be used for tiles in this tileset.
  */
@@ -592,31 +622,41 @@ inline SharedTileset Tileset::sharedPointer() const
 }
 
 /**
- * Sets whether this tileset was loaded successfully. This variable is true by
- * default, but it can be set to false to indicate a failed attempt at loading
- * an external tileset.
+ * Sets the status of this tileset.
  */
-inline void Tileset::setLoaded(bool loaded)
+inline void Tileset::setStatus(LoadingStatus status)
 {
-    mLoaded = loaded;
+    mStatus = status;
 }
 
 /**
- * Returns whether this tileset was loaded succesfully. Only valid for
- * external tilesets (fileName() != empty).
+ * Sets the loading status of this tileset's image.
  */
-inline bool Tileset::loaded() const
+inline void Tileset::setImageStatus(LoadingStatus status)
 {
-    return mLoaded;
+    mImageReference.status = status;
 }
 
 /**
- * Returns whether the image used by this tileset was loaded succesfully. Only
- * valid for tilesets based on a single image (imageSource() != empty).
+ * Returns the loading status of this tileset.
+ *
+ * Only valid for external tilesets (fileName() != empty).
  */
-inline bool Tileset::imageLoaded() const
+inline LoadingStatus Tileset::status() const
 {
-    return mImageReference.loaded;
+    return mStatus;
+}
+
+/**
+ * Returns the loading status of this tileset's image.
+ *
+ * Only valid for tilesets based on a single image (imageSource() != empty).
+ */
+inline LoadingStatus Tileset::imageStatus() const
+{
+    return mImageReference.status;
 }
 
 } // namespace Tiled
+
+Q_DECLARE_METATYPE(Tiled::SharedTileset)
